;------------------------------------------------------------------------
;                               Iterator ANY?
;
; -- procedure+: any? PRED COLLECTION
;       Searches for the first element in the collection satisfying a
;       given predicate
;       That is, the procedure applies PRED to every element of the
;       COLLECTION in turn.
;       The first element for which PRED returns non-#f stops the iteration;
;       the value of the predicate is returned.
;       If none of the elements of the COLLECTION satisfy the predicate,
;       the return value from the procedure is #f
;       COLLECTION can be a list, a vector, a string, or an input port.

;;; guile extension by unknown_lamer
(define-module (oleg any)
  #:export (any?))

(define (any? <pred?> coll)
  (cond
    ((list? coll)
      (let loop ((curr-l coll))
        (if (null? curr-l) #f
          (or (<pred?> (car curr-l)) (loop (cdr curr-l))))))
          
    ((vector? coll)
      (let ((len (vector-length coll)))
       (let loop ((i 0))
        (if (>= i len) #f
          (or (<pred?> (vector-ref coll i)) (loop (+ 1 i)))))))

    ((string? coll)
      (let ((len (string-length coll)))
       (let loop ((i 0))
        (if (>= i len) #f
          (or (<pred?> (string-ref coll i)) (loop (+ 1 i)))))))

    ((input-port? coll)
      (let loop ((c (read-char coll)))
        (if (eof-object? c) #f
          (or (<pred?> c) (loop (read-char coll))))))

    (else (error "any? on an invalid collection"))))

;-----------------------------------------------------------------------------
;   This is a test driver for any?, to make sure it
;                       really works as intended

; (define (cout . args)
;   (for-each (lambda (x)
;               (if (procedure? x) (x) (display x)))
;             args))

; (define (test-any?-driver pred? coll expected-result)
;   (display "testing ")
;   (write `(any? ,pred? ,coll))
;   (let ((res (any? pred? coll)))
;     (if (not (eqv? res expected-result))
;         (error "computed result " res "differs from the expected one "
;                expected-result)
;         (cout " yields the expected result " res newline))))

;                                 ; some testing predicates
; (define (eq-a? x) (if (char=? x #\a) x #f))
; (define (not-a? x) (if (eqv? x #\a) #f x))
; (define (gt1? x) (if (> x 1) x #f))
  
; (cout "finding an element in a list" newline newline)
; (test-any?-driver gt1? '(1 2 3 4 5) 2)
; (test-any?-driver gt1? '(1 1 1 1 1) #f)
; (test-any?-driver gt1? '(4 1 1 1 1) 4)
; (test-any?-driver gt1? '(4 5 6 1 9) 4)
; (test-any?-driver gt1? '(-4 -5 -6 1 9) 9)
; (test-any?-driver eq-a? '(#\b #\c #\a 2) #\a)
; (test-any?-driver eq-a? '(#\b #\c #\g #\k) #f)
; (test-any?-driver not-a? '(#\a #\c #\a 1) #\c)
  
; (cout "finding an element in a vector" newline newline)
; (test-any?-driver gt1? '#(1 2 3 4 5) 2)
; (test-any?-driver gt1? '#(1 1 1 1 1) #f)
; (test-any?-driver gt1? '#(4 1 1 1 1) 4)
; (test-any?-driver gt1? '#(4 5 6 1 9) 4)
; (test-any?-driver gt1? '#(-4 -5 -6 1 9) 9)
; (test-any?-driver eq-a? '#(#\b #\c #\a #\k) #\a)
; (test-any?-driver eq-a? '#(#\b #\c #\k #\k) #f)
; (test-any?-driver not-a? '#(#\a 1 #\a #\k) 1)
  
; (cout "finding an element in a string" newline newline)
; (test-any?-driver eq-a? "bc ak" #\a)
; (test-any?-driver eq-a? "bc bk" #f)
; (test-any?-driver not-a? "bc dk" #\b)
; (test-any?-driver eq-a? "aaaaa" #\a)
; (test-any?-driver not-a? "aaaaa" #f)
; (test-any?-driver not-a? "aaaab" #\b)

; (cout "finding an element in an input port" newline newline)
; (with-input-from-string "bc ak"
;  (lambda () (test-any?-driver eq-a? (current-input-port) #\a)))
; (with-input-from-string "bc bk"
;  (lambda () (test-any?-driver eq-a? (current-input-port) #f)))
; (with-input-from-string "bc ,k"
;  (lambda () (test-any?-driver not-a? (current-input-port) #\b)))
; (with-input-from-string "aaaaa"
;  (lambda () (test-any?-driver eq-a? (current-input-port) #\a)))
; (with-input-from-string "aaaaa"
;  (lambda () (test-any?-driver not-a? (current-input-port) #f)))
; (with-input-from-string "aaaab"
;  (lambda () (test-any?-driver not-a? (current-input-port) #\b)))

; (cout "done" newline newline)
